
/*
 * LpInv: Prototype Program Analyzer for Linear Systems using 
 *        Linear Programming. 
 * Author: Sriram Sankanarayanan <srirams@theory.stanford.edu> 
 * Copyright (c) 2005 Sriram Sankaranarayanan.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *        
 */


#ifndef D__EQN__H_
#define D__EQN__H_

#include <iostream>
#include <map>
#include "var-info.h"
#include "ppl.hh"
#include "PolyUtils.h"

using namespace std;
using namespace Parma_Polyhedra_Library;
using namespace Parma_Polyhedra_Library::IO_Operators;

class Eqn{

  private:

   /*
    * _n is the number of dimensions
    * _m is a map that takes non-zero dimensions to their
    *     coefficients
    *
    * _times is the # of times the template has been evolved
    * 
    */
   
   int _n;  
   map<int,long> _m;
   
   int _times; 
   
   bool _frozen;

   void parse_generator(Generator const & g);

   void parse_constraint(Constraint const & cc, int scale);
   
  public:

   /*
    * Constructors
    */
   
   Eqn(int n):_n(n),_times(0),_frozen(false){}

   Eqn( int n, Generator const & g):_n(n), _times(0), _frozen(false){
      parse_generator(g);
   }
   
   Eqn( int n, Constraint const & cc, int scale=1):_n(n), _times(0), _frozen(false){
      parse_constraint(cc, scale);
   }
   
   
   int get_dimension() const{
      return _n;
   }
   
   
   /*
    *  Is it still evolving, or are we ready to widen?
    */
   bool is_frozen() const {
      return _frozen;
   }
   bool freeze(bool frozen){
      _frozen=frozen;
      return _frozen;
   }

   
   bool is_evolving() const;

   void increase_times(){
      _times++;
   }
   /* 
    * accessors
    */

   long operator()(int k) const;
   long & operator[](int k);

   
   /*
    * A simple print routine
    */

   void print(ostream & out) const;
   
   void print_using(ostream & out, var_info * f) const;

   bool positive_matches(Eqn const & what) const;

   void scale_up(int den);

   /* Check if there are a,b > 0 s.t.  a * (c) = b * this */

   bool compare(Eqn const & c , long & a, long & b) const;

   bool is_zero() const;
   
};

ostream & operator<< ( ostream & out, Eqn const & what);

#endif
