
/*
 * LpInv: Prototype Program Analyzer for Linear Systems using 
 *        Linear Programming. 
 * Author: Sriram Sankanarayanan <srirams@theory.stanford.edu> 
 * Copyright (c) 2005 Sriram Sankaranarayanan.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *        
 */

#ifndef D__EQN_UTILITIES__H_
#define D__EQN_UTILITIES__H_

#include <iostream>
#include <map>
#include "ppl.hh"
#include "TrSys.h"
#include "GenPool.h"
#include "Eqn.h"
#include "EquationTemplate.h"
using namespace std;
using namespace Parma_Polyhedra_Library;
using namespace Parma_Polyhedra_Library::IO_Operators;

/*
 * Utilities for 
 *   1. Building a EquationTemplate from a pool of generators
 *
 *   2. Building and solving  a feasibility/optimization problem
 *      from a pool of generators for the post computation/widening.
 *
 */

void buildEquationTemplate( GenPool const & gp, EquationTemplate & result);
void buildEquationTemplate( C_Polyhedron const & p, EquationTemplate & result);

void buildLProblem ( EquationTemplate const & pre, C_Polyhedron const & guard,  LProblem & result);
void buildLProblem ( EquationTemplate const & pre,  LProblem & result);
void buildLProblem(C_Polyhedron const & p, LProblem & result);


void handle_constraint( Constraint const & cc, int col_num, int n, LProblem & result);
void handle_eqn( Eqn const & eq, int col_num, int n, LProblem & result);
long handle_with_exception(Coefficient const & c);

long evaluate(Linear_Expression const & ll, Eqn const & what);

void update_map(int n, Eqn const & what, TrRel const & trs, Eqn & result);

/* optimize the constant in c w.r.t. lpr and add to what.
 * if blind is set to true, then add without checking  for duplication 
 */

void optimize_and_add(Eqn & c, EquationTemplate & what , LProblem & lpr, bool blind=false);
/*
 * optimize the constant coefficient, in effect fitting the strongest assertion.
 *   return < val,true> -- everything OK, the optimum is val
 *   return < 0,false> -- infeasible , no such assertion is possible
 *   return < -1, false> -- unbounded, the premises are empty (for post computation this signals that
 *                                        guard /\ poly is empty).
 */

pair<long,bool> optimize_coefficient(EquationTemplate & pre, Eqn & eq, TrRel const & trs, LProblem & lpr);


bool optimize_coefficient_fractional (EquationTemplate & pre, Eqn & eq, TrRel const & trs, LProblem & lpr, pair<long,long> & result, bool preopt=false);

void post_computation( EquationTemplate & pre, EquationTemplate & post, TrRel const & trs);
//void post_computation( EquationTemplate & pre, TrRel const & trs);

pair<long,bool> optimize_coefficient(Eqn & c, LProblem & result, long offset =0);

bool optimize_coefficient_fractional(Eqn & c, LProblem & result, long offset, pair<long,long> & res);


void add_extra_relations(EquationTemplate & what, LProblem & lpr);
void add_extra_relations(EquationTemplate & what);

void enrich_pair(EquationTemplate & pre, EquationTemplate & post, TrRel const & trs, LProblem & lpr);
void enrich_pair(EquationTemplate & pre, EquationTemplate & post, TrRel const & trs);

/*
 * The case for self-looping transitions needs to be handled differently
 */

void enrich_pair(EquationTemplate & pre,  TrRel const & trs, LProblem & lpr);
void enrich_pair(EquationTemplate & pre,  TrRel const & trs);

/* 
 *  Quick optimization by just comparing stuff syntactically
 *  Post-comment: This line of investigation has been disabled.
 *
 */ 

bool quick_optimize_fractional(Eqn &c, EquationTemplate & pre, pair<long,long> & result);

#endif
