
/*
 * LpInv: Prototype Program Analyzer for Linear Systems using 
 *        Linear Programming. 
 * Author: Sriram Sankanarayanan <srirams@theory.stanford.edu> 
 * Copyright (c) 2005 Sriram Sankaranarayanan.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *        
 */


#ifndef D__EVOLVING_GENS__H_
#define D__EVOLVING_GENS__H_
#include <iostream>
#include <vector>
#include <map>
#include <ppl.hh>
#include "Rational.h"
#include "var-info.h"

#include "funcs.h"
#include "PolyUtils.h"
using namespace std;
using namespace Parma_Polyhedra_Library;
using namespace Parma_Polyhedra_Library::IO_Operators;


class EvolvingGens{

  private:

   int _n;
   Generator _g;
   bool _valid;
   int _times;
   
   void adjust_constant_coefficient(C_Polyhedron const & ref);
   
  public:

   EvolvingGens( int n, Generator const & g);
   EvolvingGens( int n, Generator const & g, C_Polyhedron const & ref);

   void refine_with(C_Polyhedron const & ref){
      adjust_constant_coefficient(ref);
   }
   
   Generator & obtain_generator(){
      return _g;
   }
   
   Generator const & obtain_generator() const{
      return _g;
   }

   
   bool is_valid() const{
      return _valid;
   }


   void print(ostream & out) const;
   
};

ostream & operator<<(ostream & out, EvolvingGens const & ev);

#endif
