
/*
 * LpInv: Prototype Program Analyzer for Linear Systems using 
 *        Linear Programming. 
 * Author: Sriram Sankanarayanan <srirams@theory.stanford.edu> 
 * Copyright (c) 2005 Sriram Sankaranarayanan.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *        
 */


#ifndef D__GEN_POOL__H_
#define D__GEN_POOL__H_

#include <iostream>
#include <vector>
#include <map>
#include "Rational.h"
#include "var-info.h"
#include <ppl.hh>
#include "funcs.h"
#include "PolyUtils.h"
#include "EvolvingGens.h"
using namespace std;
using namespace Parma_Polyhedra_Library;
using namespace Parma_Polyhedra_Library::IO_Operators;

class GenPool{

  private:

   int _n;
   vector<EvolvingGens> _vg;
   C_Polyhedron _poly;
   
   void initialize_vg_with(C_Polyhedron const & init);
   void compute_polyhedron();
   void add_evolving_gen(EvolvingGens const & eg);
   
  public:

   GenPool(int n, C_Polyhedron const & init ):_n(n), _poly(init) {

      if ( _poly.is_universe())
         return;
      
      // initialize the evolving generators
      initialize_vg_with(init);
   }

   
   C_Polyhedron const & obtain_polyhedron() const {
      return _poly;
   }

   C_Polyhedron & obtain_polyhedron() {
      return _poly;
   }

   void narrowing_assign(C_Polyhedron const & next);

   void print(ostream & out) const;

   vector<EvolvingGens> const & obtain_generators() const{
      return _vg;
   }

   int obtain_dimensions() const{
      return _n;
   }
};

ostream & operator<<(ostream & out, GenPool const & gp);

#endif
