
/*
 * LpInv: Prototype Program Analyzer for Linear Systems using 
 *        Linear Programming. 
 * Author: Sriram Sankanarayanan <srirams@theory.stanford.edu> 
 * Copyright (c) 2005 Sriram Sankaranarayanan.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *        
 */


#include "myassertions.h"
#include "GenPoolMap.h"

GenPoolMap::GenPoolMap (IMap const & im): _vloc(im.location_vector()){
   vector<TrLoc>::const_iterator vi;
   
   for (vi=_vloc.begin(); vi != _vloc.end(); ++vi){
      string const & name = vi -> get_name();
      GenPool npool(vi->get_dimension(), im(name));
      _m.insert(StrGenPair(name,npool));
   }
   
   
}

C_Polyhedron const & GenPoolMap::operator() (string const & loc) const{

   PRECONDITION( entry_exists(loc) , " Asked to fetch for a non-existent location "<<loc);

   GenPool const & x = (_m.find(loc))->second;

   return x.obtain_polyhedron();
}


GenPool & GenPoolMap::operator[] (string const & what){
   PRECONDITION( entry_exists(what) , " Asked to fetch for a non-existent location "<<what);
   return (_m.find(what))->second;
}



void GenPoolMap::print(ostream & out) const{

   StrGenMap::const_iterator vi;
   for (vi=_m.begin(); vi!= _m.end(); ++vi){
      out << (vi->first) << "[["<<endl;

      out << (vi -> second) << endl ;
      out << "]]"<<endl;
   }
   
}

ostream & operator<< (ostream & out, GenPoolMap const & gpm){
   gpm.print(out);
   return out;
}



void GenPoolMap::update_imap(IMap & result) const {
   vector<TrLoc>::const_iterator vi;

   for (vi=_vloc.begin(); vi != _vloc.end(); ++vi){
      string const & name = vi -> get_name();

      result[name] = operator()(name);

   }

   return;

}


void GenPoolMap::narrowing_assign(IMap const & what){
      vector<TrLoc>::const_iterator vi;

   for (vi=_vloc.begin(); vi != _vloc.end(); ++vi){
      string const & name = vi -> get_name();
      GenPool & gp = _m.find(name)->second;
      C_Polyhedron const & pp  = what(name);

      gp.narrowing_assign(pp);
   }

   return;
}
