
/*
 * LpInv: Prototype Program Analyzer for Linear Systems using 
 *        Linear Programming. 
 * Author: Sriram Sankanarayanan <srirams@theory.stanford.edu> 
 * Copyright (c) 2005 Sriram Sankaranarayanan.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *        
 */



#ifndef D__GENPOOL_MAP__H_
#define D__GENPOOL_MAP__H_


#include <iostream>
#include <vector>
#include <map>
#include "Rational.h"
#include "var-info.h"
#include <ppl.hh>
#include "TrLoc.h"
#include "IMap.h"
#include "EvolvingGens.h"
#include "GenPool.h"

using namespace std;
using namespace Parma_Polyhedra_Library;
using namespace Parma_Polyhedra_Library::IO_Operators;


/*
 *
 * Once widening is started, storing a generator pool 
 * and being able to compute post on generator pools makes
 * trying out different widenings easier. Most code needs to 
 * mimic the code in IMap.h --- unfortunately.
 *
 */




typedef pair<string, GenPool> StrGenPair;
typedef map<string, GenPool> StrGenMap;

class GenPoolMap{
  private:
   vector<TrLoc> const & _vloc;
   StrGenMap _m;
   
  public:
   
   GenPoolMap(IMap const & im);
   

   bool entry_exists(TrLoc const & loc) const{
      return (entry_exists(loc.get_name()));
   }
   
   bool entry_exists(string const & what) const{
      return _m.find(what) != _m.end();
   }

   C_Polyhedron const & operator() (TrLoc const & loc) const{
      return operator()(loc.get_name());
   }

   C_Polyhedron const & operator() (string const & loc) const;

   GenPool & operator[] (TrLoc const & loc) {
      return operator[] (loc.get_name());
   }
   
   GenPool & operator[] (string const & what);

   void print(ostream & out) const;

   void update_imap(IMap & result) const;

   //
   // narrow this map w.r.t. what 
   //
   
   void narrowing_assign(IMap const & what);

   
   
};

ostream & operator << (ostream & out, GenPoolMap const & what);


#endif
