
/*
 * LpInv: Prototype Program Analyzer for Linear Systems using 
 *        Linear Programming. 
 * Author: Sriram Sankanarayanan <srirams@theory.stanford.edu> 
 * Copyright (c) 2005 Sriram Sankaranarayanan.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *        
 */


#ifndef D__IMAP__H_
#define D__IMAP__H_

#include <iostream>
#include <vector>
#include <map>
#include "Rational.h"
#include "var-info.h"
#include <ppl.hh>
#include "TrLoc.h"
#include "TrRel.h"


using namespace std;
using namespace Parma_Polyhedra_Library;
using namespace Parma_Polyhedra_Library::IO_Operators;

/*
 * A class to store dual invariant assertion maps
 *
 */

typedef pair<string,C_Polyhedron> StrPolyPair;
typedef map<string,C_Polyhedron> StrPolyMap;
typedef vector<Generator> GenVector;
typedef pair<string,GenVector> StrFramePair;
typedef map<string,GenVector> StrFrameMap;

class IMap{
  private:
   vector<TrLoc> const & _vloc; 
   StrPolyMap _m;
   StrFrameMap _frames;

   int _dim;
   
   void add_initial_generators(GenVector & v);
   GenVector & get_frames( string const & name);
   
  public:
   
   // create a  map
   IMap(vector<TrLoc> const & vloc, bool _dual=true);


   vector<TrLoc> const & location_vector() const{
      return _vloc;
   }

   // obtain the polyhedron corresponding to a location

   bool entry_exists(TrLoc const & loc) const;
   bool entry_exists(string const & what) const;

   
   C_Polyhedron const & operator()( TrLoc const & loc) const;
   C_Polyhedron const & operator()( string const & what) const;


   C_Polyhedron & operator[]( TrLoc const & loc);
   C_Polyhedron & operator[]( string const & what);

   // check two maps for containment

   bool contains(IMap const & what) const;

   // print

   void print(ostream & out) const;
   
   // primalize the map

   void primalize(IMap  & result) const;

   //
   // narrow a map w.r.t its next iterate
   // result map has to be contained in this map
   //
   
   void narrow_with( IMap & result) const; 

   void widening_assign(IMap const & what);
   
   IMap & operator= (IMap const & what);

   void print_using(ostream & out, var_info * f)const;

   void check(vector<TrRel> const & vrel) const;
   
   void propagation_assign( vector<TrRel> const & vrel , IMap & result);
   
};

ostream & operator<< (ostream & out, IMap const & what);




#endif
