
/*
 * LpInv: Prototype Program Analyzer for Linear Systems using 
 *        Linear Programming. 
 * Author: Sriram Sankanarayanan <srirams@theory.stanford.edu> 
 * Copyright (c) 2005 Sriram Sankaranarayanan.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *        
 */


#ifndef D__LIN__EXPR__H_
#define D__LIN__EXPR__H_

// Linear expressions

//#include <saclib.h>
#include "global_types.h"
#include "Rational.h"
#include "var-info.h"
#include <iostream.h>
#include <ppl.hh>

using namespace std;
using namespace Parma_Polyhedra_Library;
using namespace Parma_Polyhedra_Library::IO_Operators;

class LinExpr{
  protected:
   int n;

   vector<Rational>  lin;

   var_info * f;
   int count;
   
   void initialize(int n, var_info * f);
   void clear_out();
   
  public:
   LinExpr();
   LinExpr(int n, var_info * f);
   LinExpr(LinExpr const & ll);
   
   
   void init_set(int n, var_info * f);

   // LinExpr(int n); // Can be added later if required

   LinExpr  operator + ( LinExpr const & p1) const; // Addition
   
   LinExpr&  operator += (LinExpr const & p1); // Addition to self

   LinExpr  operator - ( LinExpr const & p1) const; // Addition
   
   LinExpr&  operator -= (LinExpr const & p1); // Addition to self

   LinExpr&  operator= (LinExpr const & p1); // Assignment

   LinExpr& operator*= (int i);

   LinExpr& operator*= (Rational const & i);

   // 
   // checks if there is a factor c such that this - c.t
   // 
   bool operator== (LinExpr const & t) const;

   int get_count() const { return count;}
   
   int reset_count();
   int count_up();
   int count_down();

   bool check_variable_compatibility (var_info const * f1) const{
      return (f1==f);
   }
   
   Rational & operator[](int i);

   Rational operator() (int i) const;
   
   int get_dim() const;
   var_info * get_info() const;

   // disabled
   
   // vector<Rational> & get_array();

   bool is_constant() const;
   bool is_zero()const;
   ~LinExpr();

   int get_denominator_lcm() const;
   int get_numerator_gcd()const;
   bool equiv(LinExpr const & l1, Rational &factor) const;
   
   Linear_Expression to_lin_expression() const;
   
   Constraint get_constraint(int ineq_type) const;

   Rational evaluate(Generator const & g);
   void print(ostream & os) const;
};

ostream& operator<< (ostream & os, LinExpr const & expr); // to print my beautiful Lin Exprs

LinExpr operator* (Rational const & i, LinExpr const & p1);
LinExpr operator* (LinExpr const & p1, Rational const & i);

LinExpr operator* (int i, LinExpr const & p1);


#endif
