
/*
 * LpInv: Prototype Program Analyzer for Linear Systems using 
 *        Linear Programming. 
 * Author: Sriram Sankanarayanan <srirams@theory.stanford.edu> 
 * Copyright (c) 2005 Sriram Sankaranarayanan.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *        
 */


#ifndef D__LOC_TEMPLATE_MAP__H_
#define D__LOC_TEMPLATE_MAP__H_

#include <iostream>
#include <map>
#include <string>
#include "var-info.h"
#include "Rational.h"
#include "TrLoc.h"
#include "TrRel.h"
#include "TrSys.h"
#include "IMap.h"

#include "LProblem.h"
#include "Eqn.h"
#include "EquationTemplate.h"
#include "EqnUtilities.h"
#include "TemplateBuilder.h"

typedef map<string,EquationTemplate> StrTempMap;
typedef pair<string, EquationTemplate> StrTempPair;

class LocTemplateMap{

  private:
   
   int _n;
   var_info * _f;
   StrTempMap _m;
   vector<TrLoc> const & _vloc;
   vector<TrRel> const & _vrel;
   
   /*
    * methods for the initial creation of a 
    * set of constraints
    */

   void populate_map();

   void populate_map(vector<TemplateBuilder> & vtb);
   void populate_map(EquationTemplate & what);
   
 
   void initialize(TrLoc const & lc, EquationTemplate & what);
   
   // may be implemented later
   
   void enrich_map_once();
   void add_default_pairs(C_Polyhedron const & init, EquationTemplate & what);
   
  public:

   /* 
    * Construct a map and populate with octagons etc..
    */ 
   LocTemplateMap(int n, var_info * f, vector<TrLoc> const & vloc , vector<TrRel> const & vrel);

   /*
    * Construct a map with a vector of templates that I am providing
    */

   LocTemplateMap(int n, var_info * f, \
                  vector<TrLoc> const & vloc ,\
                  vector<TrRel> const & vrel ,\
                  vector<TemplateBuilder> & vtb);
    
   

   /*
    * Construct the map with actual eqns from ineqs
    */

   LocTemplateMap(int n, var_info *f, \
                  vector<TrLoc> const & vloc, \
                  vector<TrRel> const & vrel, \
                  EquationTemplate & what);
   
   
   /*
    * update the "postlocation" template through post
    * returns true if there is a change 
    */

   
   bool post(TrRel const & what);

   void compute_invariants();
   

   void print( ostream & out ) const;

   void fill_imap(IMap & result) const;

   int get_total_template_size() const;
   
   
};

ostream & operator<< (ostream & out, LocTemplateMap const & ltm);


#endif
