
/*
 * LpInv: Prototype Program Analyzer for Linear Systems using 
 *        Linear Programming. 
 * Author: Sriram Sankanarayanan <srirams@theory.stanford.edu> 
 * Copyright (c) 2005 Sriram Sankaranarayanan.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *        
 */



#include "myassertions.h"
#include "OneParameterConstraint.h"


bool OneParameterConstraint::geq(Rational const & r){

   // if empty nothing to be done

   if (_emp) return false;

   // if universal or no prior lower limit then set lower limit to r
   if (_univ || !_lower) {
      _univ=false;
      _lower=true;
      _emp=false;
      _l = r;
      return true;
   }

   // else if r is greater than the lower limit than
   // set the lower limit to r
   
   if (r >= _l)
      _l=r;
   
   // if it has an upper limit which is now
   // less than the lower limit then it is inconsistent
   
   if ( _upper && (_u < _l)){
      _emp=true;
      _lower=false;
      _upper=false;
      return false;
   }

   // __CLASS_INVARIANT_CHECK();
   
   return true;
   
}



bool OneParameterConstraint::ge( Rational const & r){
   //ftb I am relaxing this to geq
   PRECONDITION( false, " Not implemented handling of strict inequalities as yet ");
   
   return geq(r);
}


bool OneParameterConstraint::eq(Rational const & r){

   // if empty nothing to be done

   if (_emp) return false;

   // if universal then set both lower and upper limits

   if (_univ){
      _univ=false;
      _lower=true;
      _upper=true;
      _u=r;
      _l=r;
      return true;
   }

   if (_lower && _l > r) {
      _emp=true;
      _univ=_lower=_upper=false;
      return false;
   }
   
   
   if (_upper && _u < r) {
      _emp=true;
      _univ=_lower=_upper=false;
      return false;
   }

   _lower=_upper=true;
   _univ=_emp=false;

   _u=r;
   _l=r;

   return true;

}


