
/*
 * LpInv: Prototype Program Analyzer for Linear Systems using 
 *        Linear Programming. 
 * Author: Sriram Sankanarayanan <srirams@theory.stanford.edu> 
 * Copyright (c) 2005 Sriram Sankaranarayanan.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *        
 */


#ifndef D__ONEPARAMETERCONSTRAINT__H_
#define D__ONEPARAMETERCONSTRAINT__H_

#include <iostream>
#include "Rational.h"

using namespace std;

class OneParameterConstraint{

  private:
   bool _univ;
   bool _emp;

   bool _upper;
   Rational _u;

   bool _lower;
   Rational _l;
   

  public:

  OneParameterConstraint::OneParameterConstraint():_univ(true),
     _emp(false),
     _upper(false),
     _lower(false){}

   bool is_universal() const{
      return _univ;
   }

   bool is_empty() const{
      return _emp;
   }

   bool is_equal_to() const{
      return (_upper && _lower && _u==_l);
   }

   bool has_upper_limit() const{
      return _upper;
   }
   bool has_lower_limit() const{
      return _lower;
   }

   Rational upper_limit() const{
      return _u;
   }

   Rational lower_limit() const{
      return _l;
   }

   bool geq(Rational const & r);

   //  bool leq(Rational r);

   bool ge(Rational const & r);

   // bool le (Rational r);

   bool eq (Rational const & r);
   

   
   
};

#endif
