
/*
 * LpInv: Prototype Program Analyzer for Linear Systems using 
 *        Linear Programming. 
 * Author: Sriram Sankanarayanan <srirams@theory.stanford.edu> 
 * Copyright (c) 2005 Sriram Sankaranarayanan.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *        
 */



// Some utilities for handling polyhedra

#ifndef __POLYUTILS__H_
#define __POLYUTILS__H_
#include <iostream.h>
#include "var-info.h"
#include "LinExpr.h"
#include <ppl.hh>

using namespace std;
using namespace Parma_Polyhedra_Library;
using namespace Parma_Polyhedra_Library::IO_Operators;


ostream & print_polyhedron(ostream& in, C_Polyhedron const & np, const var_info * f);

ostream & print_lin_expression(ostream & in, Linear_Expression const & lp, const var_info * f);

ostream & print_constraint( ostream & in, Constraint const & cc, var_info * f);


void print_generators(ostream & out, Generator_System const & gg, var_info * f);

int handle_integers(Coefficient const & t);

bool handle_integers(Coefficient const & t , int & result);

C_Polyhedron * add_dimensions_initial(C_Polyhedron * p, int what); // add what dimensions to p at the beginning

void dualize(C_Polyhedron const & what, C_Polyhedron & result);

void primal(C_Polyhedron const & what, C_Polyhedron & result);

void set_up_affine_transform(int n, Constraint const & cc, Linear_Expression & left, Linear_Expression & right);

void test_and_add_generator(int n, Generator const & a, C_Polyhedron const & test, C_Polyhedron & result);

// result is assumed to be narrower than what

void H79_narrow( C_Polyhedron & result,  C_Polyhedron const & what );

void H79_narrow(C_Polyhedron & result, C_Polyhedron const & what, vector<Generator> & frames );


int compute_size( Constraint_System const & cs);

#endif

