
/*
 * LpInv: Prototype Program Analyzer for Linear Systems using 
 *        Linear Programming. 
 * Author: Sriram Sankanarayanan <srirams@theory.stanford.edu> 
 * Copyright (c) 2005 Sriram Sankaranarayanan.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *        
 */


#ifndef D__RESOURCE_MONITOR__H_
#define D__RESOURCE_MONITOR__H_

#include <iostream>
#include <vector>

/*
 * Monitors maximum and average memory usage.
 * The process will call the take_reading()
 * method to take a reading.
 *
 */

using namespace std;

class ResourceMonitor{

  private:
   vector<long> _vreads;

  public:

   ResourceMonitor(){}

   int get_num_readings() const{
      return _vreads.size();
   }

   void take_a_reading();

   long get_average() const{

      if (_vreads.size()==0) return 0;
      
      vector<long>::const_iterator vi;
      long count=0;

      for (vi=_vreads.begin(); vi!= _vreads.end(); ++vi){
         count+= (*vi);
      }

      return (count/_vreads.size());
   }

   long get_maximum() const{
      
      if (_vreads.size()==0) return 0;
      
      vector<long>::const_iterator vi;

      long max=*(_vreads.begin());
      
      for (vi=_vreads.begin(); vi!= _vreads.end(); ++vi){
         if ((*vi) > max)
            max= (*vi);
      }

      return max;
   }

};



#endif
