
/*
 * LpInv: Prototype Program Analyzer for Linear Systems using 
 *        Linear Programming. 
 * Author: Sriram Sankanarayanan <srirams@theory.stanford.edu> 
 * Copyright (c) 2005 Sriram Sankaranarayanan.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *        
 */

#ifndef D__TRLOCATION__H_
#define D__TRLOCATION__H_

#include "TrLocImpl.h"

using namespace std;
using namespace Parma_Polyhedra_Library;
using namespace Parma_Polyhedra_Library::IO_Operators;

class TrRel;

class TrLoc{
  private:

   TrLocImpl * _trl;
   
  public:

   TrLoc(int n,var_info * f, string name, C_Polyhedron const &init){
      _trl=new TrLocImpl(n,f,name,init);
   }

   
   // accessors
   int get_dimension() const{
      return _trl->get_dimension();
   }

   var_info * get_var_info() const{
      return _trl->get_var_info();
   }
  var_info * get_dual_var_info() const{
     return _trl->get_dual_var_info();
  }
   string const & get_name() const{
      return _trl->get_name();
   }

   C_Polyhedron const & get_init() const{
      return _trl->get_init();
   }

   C_Polyhedron const & get_dinit() const{
      return _trl->get_dinit();
   }

   LProblem const & get_lproblem_ref()const{
      return _trl->get_lproblem_ref();
   }
   
   bool is_init_empty() const{
      return _trl->is_init_empty();
   }
   
   // print

   void print( ostream & out) const {
      _trl->print(out);
   }

   // comparison of locations

   bool operator==(TrLoc const & what) const{
      return _trl->get_name()==what.get_name();
   }


};


ostream & operator<<(ostream & out, TrLoc const & what);


#endif
