
/*
 * LpInv: Prototype Program Analyzer for Linear Systems using 
 *        Linear Programming. 
 * Author: Sriram Sankanarayanan <srirams@theory.stanford.edu> 
 * Copyright (c) 2005 Sriram Sankaranarayanan.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *        
 */

#include "myassertions.h"
#include "funcs.h"
#include "TrLocImpl.h"
#include "EqnUtilities.h"

using namespace std;
using namespace Parma_Polyhedra_Library;
using namespace Parma_Polyhedra_Library::IO_Operators;

extern bool lpinv;

TrLocImpl::TrLocImpl(int n,var_info * f, string name, C_Polyhedron const &init):
      _n(n),
      _f(f),
      _name(name),
      _init(init),
      _dinit(_n+1,UNIVERSE),
      _lpr(1,1){
   

   // create a dual var_info
   _fd=new var_info();
   // and populate it
   
   int i;
   string temp;

   // check if init is empty, if not,
   // build a linear program to help
   // things along
   if (lpinv){
      if (init.is_empty())
         _init_empty=true;
      else{
         _init_empty=false;
         buildLProblem(_init,_lpr);
      }
   }
   
   for (i=0;i<_n;++i){
      temp= _name+"_c_"+f->get_name(i);
      _fd->insert(temp.c_str());
   }
   temp= _name+"_"+"d";
   _fd->insert(temp.c_str());
   //dualize(_init,_dinit);
}


void TrLocImpl::print(ostream & out) const{

   //
   // print the location's contents
   //

   out<<"Location : "<<_name<<endl;
   out<<"Initial Condition [["<<endl;
   print_polyhedron(out,_init, _f);
   out<<"]]"<<endl;

   return;
   
}

ostream & operator<< (ostream & out, TrLocImpl const & what){
   what.print(out);
   return out;
}



LProblem const & TrLocImpl::get_lproblem_ref() const{

   PRECONDITION( !_init_empty, " Asking for a LProblem from a location with empty init? Why?");

   return (_lpr);
}


