
/*
 * LpInv: Prototype Program Analyzer for Linear Systems using 
 *        Linear Programming. 
 * Author: Sriram Sankanarayanan <srirams@theory.stanford.edu> 
 * Copyright (c) 2005 Sriram Sankaranarayanan.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *        
 */

#ifndef D__TRLOCATIONIMPLEMENTATION__H_
#define D__TRLOCATIONIMPLEMENTATION__H_

#include <iostream>
#include <string>
#include <ppl.hh>
#include "var-info.h"
#include "Rational.h"
#include "PolyUtils.h"
#include "LProblem.h"

using namespace std;
using namespace Parma_Polyhedra_Library;
using namespace Parma_Polyhedra_Library::IO_Operators;

class TrRel;

class TrLocImpl{
  private:

   int _n;
   var_info * _f, * _fd;
   string _name;
   C_Polyhedron _init;
   C_Polyhedron _dinit;

   LProblem _lpr;
   
   bool _init_empty;
   
  public:

   TrLocImpl(int n,var_info * f, string name, C_Polyhedron const &init);

   
   // accessors
   int get_dimension() const{
      return _n;
   }

   var_info * get_var_info() const{
      return _f;
   }

   var_info * get_dual_var_info() const{
      return _fd;
   }
   string const & get_name() const{
      return _name;
   }

   C_Polyhedron const & get_init() const{
      return _init;
   }

   C_Polyhedron const & get_dinit() const{
      return _dinit;
   }

   LProblem const & get_lproblem_ref()const;
   
   bool is_init_empty() const{
      return _init_empty;
   }
   
   // print

   void print( ostream & out) const;

   // comparison of locations

   bool operator==(TrLocImpl const & what) const{
      return get_name()==what.get_name();
   }


};


ostream & operator<<(ostream & out, TrLocImpl const & what);


#endif
