
/*
 * LpInv: Prototype Program Analyzer for Linear Systems using 
 *        Linear Programming. 
 * Author: Sriram Sankanarayanan <srirams@theory.stanford.edu> 
 * Copyright (c) 2005 Sriram Sankaranarayanan.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *        
 */

#ifndef D__TRRELATION__H_
#define D__TRRELATION__H_

#include <iostream>
#include <vector>
#include <map>
#include "Rational.h"
#include "var-info.h"
#include <ppl.hh>
#include "PolyUtils.h"
#include "TrLoc.h"
#include "TrRelImpl.h"

using namespace std;
using namespace Parma_Polyhedra_Library;
using namespace Parma_Polyhedra_Library::IO_Operators;

//
// A simple transition relation with a guard
// and an update relation
//

class TrRel{

  private:

   TrRelImpl * _trl;
   
  public:

   // constructor
   TrRel(TrRel const & what):_trl(what._trl){}
   
   TrRel(TrLoc & prel, TrLoc & postl,
         C_Polyhedron const & g){
      _trl= new TrRelImpl(prel,postl,g);
   }

   // require an assignment operator for overloading
   
   // access the elements
   
   TrLoc const & get_preloc() const{
      return _trl -> get_preloc();
   }

   TrLoc const & get_postloc() const{
      return _trl-> get_postloc();
   }

   string const & get_preloc_name() const{
      return _trl -> get_preloc_name();
   }

   string const & get_postloc_name() const{
      return _trl -> get_postloc_name();
   }
   
   C_Polyhedron const & get_guard() const{
      return _trl-> get_guard();
   }
   
   C_Polyhedron const & get_dual_guard() const{
      return _trl->get_dual_guard();
   }

   var_info * get_var_info() const{
      return _trl->get_var_info();
      
   }
   
   int get_dimensions() const{
      return _trl->get_dimensions();
   }
   // add an update relation

   void add_update( int i, Linear_Expression const & what){
      _trl->add_update(i,what);
   }
   void add_update( Variable v, Linear_Expression const & what){
      _trl->add_update((int) v.id(), what);
   }
  
   // Is this transition relation a petri net

   bool is_petri_net() const{
      return _trl->is_petri_net();
   }
   

   //
   // for all those variables which do not
   // have updates, add the constant update
   //

   Linear_Expression const & get_update_expression(int i) const{
      return _trl->get_update_expression(i);
   }
   
   void add_constant_updates(){
      _trl->add_constant_updates();
   }
   
   // routine to print the transition
   void print(ostream & out) const{
      _trl->print(out);
   }
   // routine to compute the post condition

   void post(C_Polyhedron const & what, C_Polyhedron & result) const{
      _trl->post(what,result);
   }

   // routine to compute the dual post

   void dual_post(C_Polyhedron const & what, C_Polyhedron & result){
      _trl->dual_post(what,result);
   }

   void print_dual(ostream & out) const{
      _trl->print_dual(out);
   }

   void compute_pre_poly(C_Polyhedron const & post_poly, C_Polyhedron & result) const{
      _trl->compute_pre_poly(post_poly,result);
   }

   void set_name(string what){
      _trl->set_name(what);
   }

   Linear_Expression const & get_primal_update_expression(int i) const{
      return _trl->get_primal_update_expression(i);
   }
      
   string const & get_name() const{
      return _trl->get_name();
   }
   
   bool self_looping() const {
      return _trl->self_looping();
   }
   
};

// the print operator.

ostream & operator<<( ostream & out, TrRel const & tt);


#endif
