
/*
 * LpInv: Prototype Program Analyzer for Linear Systems using 
 *        Linear Programming. 
 * Author: Sriram Sankanarayanan <srirams@theory.stanford.edu> 
 * Copyright (c) 2005 Sriram Sankaranarayanan.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *        
 */



#ifndef D__TRSYSTEM__H_
#define D__TRSYSTEM__H_

#include <iostream>
#include <vector>
#include <set>
#include <ppl.hh>
#include "Rational.h"
#include "var-info.h"
#include "PolyUtils.h"
#include "TrLoc.h"
#include "TrRel.h"
#include "IMap.h"

using namespace std;
using namespace Parma_Polyhedra_Library;
using namespace Parma_Polyhedra_Library::IO_Operators;

extern int _strategy_id_;

class TrSys{

  private:
   int _n;
   var_info * _f;
   vector<TrLoc> _locs;
   vector<TrRel> _rels;
   
   bool f_check_duplicates(TrLoc const & loc) const;
   
  public:
   
   TrSys(int n, var_info * f): _n(n), _f(f){}

   void add_location(TrLoc const & loc);
   void add_transition(TrRel const & rel);

   void print(ostream & out) const;

   void dual_post(IMap const & now, IMap & result);

   void compute_invariants(IMap & result, int prop_steps);
   
   void perform_initial_propagation( int k, IMap & result);

   
   void narrow_until_termination_1(IMap & now);

   
   void narrow_until_termination_2(IMap & now);

   void dispatch( int k, IMap & result);
};

ostream & operator<<( ostream & out, TrSys const & what);


#endif
