%{
#include <iostream>
#include <vector>
#include <string>
#include <ppl.hh>
#include "parse_globals.h"
#include "var-info.h"
#include "Rational.h"
#include "TrSys.h"
#include "TemplateBuilder.h"
   
using namespace std;
using namespace Parma_Polyhedra_Library;
using namespace Parma_Polyhedra_Library::IO_Operators;

 extern int yylex();
 extern int yyparse();
 

%}

%union{
   int integer;
   string * identifier;
   C_Polyhedron * poly;
   Constraint * cc;
   Linear_Expression * ll;
   ILEPair * ilp;
   vector<ILEPair> * vilp;
   TemplateBuilder * tb;
   pair<int,int> * iipair;
}


%token<integer> INT
%token<identifier> IDENT

%token TRS LOC GRD GEQ LEQ GE LE EQ END
%token ASSIGN LEADSTO WITH TRUE FALSE VARIABLE AND TEMPLATE

%type<poly> polyhedron
%type<ilp> update
%type<vilp> updates
%type<cc> linear_inequality
%type<poly>  linear_inequalities
%type<identifier> locidentifier polyvariable
%type<integer> varidentifier
%type<ll> linterm linexpression
%type<tb> template_builder
%type<iipair> template_term

%left GEQ LEQ EQ GE LE
%left '+' '-'
%left '*'
%left uminus


%start program

%%

program: header {
   process_header();
} definitions template_header locations transitions END{
   // nothing to be done after this
}
|  header {
   process_header();
} template_header locations  transitions END{
   // nothing to be done after this
}
;

header:  header VARIABLE varlist ';'  {}
| VARIABLE varlist ';' {}
;

definitions: definitions IDENT ASSIGN polyhedron ';'{
   add_poly_definition($2,$4);
}
| IDENT ASSIGN polyhedron ';' {
   add_poly_definition($1,$3);
}
;

template_header: TEMPLATE { has_templates=true; } '(' template_definitions ')' ';' {}
| { has_templates=false;}

;

template_definitions: template_builder ',' template_definitions {
   cerr<<" Template: "<<(*$1)<<endl;
   
   glb_templates.push_back((*$1));
}
| template_builder {
   cerr<<" Template: "<<(*$1)<<endl;
   glb_templates.push_back((*$1));
}
;

template_builder: template_builder '+' template_term {
   $$=$1;
   $$-> insert_pair( ($3->first), $3->second);
   delete($3);
}
| template_builder '-' template_term{
   $$=$1;
   $$-> insert_pair( -($3->first), $3->second);
   delete($3);
}
| template_term{
   
   $$= new TemplateBuilder(dimension);

   $$->insert_pair(($1->first), $1->second);

   delete($1);

}
| '-' template_term %prec uminus{

   $$= new TemplateBuilder(dimension);

   $$->insert_pair(-($2->first), $2->second);

   delete($2);
}
;

template_term: INT '*' varidentifier {
   $$=new pair<int,int>($1, $3);
}
| INT '*' place_holder {
   $$=new pair<int,int>($1, -1);
}
| varidentifier{
   $$=new pair<int,int>(1, $1);
}
| place_holder{
   $$=new pair<int,int>(1, -1);
};


place_holder: '$' IDENT { }
;

varlist:  varlist ',' IDENT {
   if (find_variable($3)!= VAR_NOT_FOUND){
      yyerror("Duplicated variable " + (*$3));
   }
   f -> insert( $3->c_str());
   delete($3);
}
| IDENT {
   if (find_variable($1)!= VAR_NOT_FOUND){
      yyerror("Duplicated variable " + (*$1));
   }
   f -> insert( $1->c_str());
   delete($1);
}
;

locations: location ';' locations  {}
| location ';' {}
;

location: LOC IDENT {

   if (location_exists($2)){
      yyerror("Duplicated Location "+ (*$2));
   }
   cout <<" Location without Polyhedron?"<<endl;
   TrLoc trl( dimension, f, (*$2), C_Polyhedron(dimension, EMPTY));
   vlocs->push_back(trl);
   delete($2);
}
| LOC IDENT WITH polyhedron {
   
   if (location_exists($2)){
      yyerror("Duplicated Location "+ (*$2));
   }
   
   TrLoc trl( dimension, f, (*$2), (*$4));
   vlocs->push_back(trl);
   
   delete($2);
   delete($4);
}
;

transitions: transition ';' transitions {}
| transition ';'
;


transition: TRS IDENT locidentifier LEADSTO locidentifier WITH GRD polyhedron  updates {

   TrRel trl(location_reference($3), location_reference($5), (*$8));
   add_updates_to_transitions(trl,$9);
   trl.add_constant_updates();

   trl.set_name(*$2);
   
   vrels->push_back(trl);

   
   delete($2);
   
   delete($3);
   delete($5);
   delete($8);
   
}
| TRS IDENT locidentifier LEADSTO locidentifier WITH  updates  {
   TrRel trl(location_reference($3), location_reference($5), C_Polyhedron(dimension, UNIVERSE));
   add_updates_to_transitions(trl,$7);
   trl.add_constant_updates();
   
   trl.set_name(*$2);
   vrels->push_back(trl);

   delete($2);
   delete($3);
   delete($5);
}
| TRS IDENT locidentifier LEADSTO locidentifier WITH  GRD polyhedron  {
   TrRel trl(location_reference($3), location_reference($5), (*$8));
   trl.add_constant_updates();

   trl.set_name(*$2);
   vrels->push_back(trl);
   
   
   delete($2);
   
   delete($3);
   delete($5);
   delete($8);
}
| TRS IDENT locidentifier LEADSTO locidentifier {
   TrRel trl(location_reference($3), location_reference($5), C_Polyhedron(dimension, UNIVERSE));
   trl.add_constant_updates();
   
   trl.set_name(*$2);
   vrels->push_back(trl);

   
   delete($2);

   delete($3);
   delete($5);
}
| TRS IDENT locidentifier WITH GRD polyhedron updates{
   TrRel trl(location_reference($3), location_reference($3), (*$6));
   add_updates_to_transitions(trl,$7);
   trl.add_constant_updates();
   trl.set_name(*$2);
   vrels->push_back(trl);

   
   
   delete($2);

   delete($3);
   delete($6);
   delete($7);
};

polyhedron: TRUE {
   
   C_Polyhedron * p = new C_Polyhedron (dimension, UNIVERSE);
   $$=p;
}
|  FALSE  {
   C_Polyhedron * p = new C_Polyhedron (dimension, EMPTY);
   $$=p;
}
| '(' linear_inequalities ')' {
   $$=$2;
}
| polyvariable AND polyhedron {

   $$=$3;
   $$->intersection_assign(get_polyhedron($1));
}
| polyvariable {
   
   C_Polyhedron * p = new C_Polyhedron (dimension, UNIVERSE);
   $$=p;
   $$->intersection_assign(get_polyhedron($1));
}
;

linear_inequalities: linear_inequalities linear_inequality {
   $$=$1;
   $$->add_constraint((*$2));
   delete($2);
   
}
| linear_inequality {
   C_Polyhedron * p = new C_Polyhedron (dimension, UNIVERSE);
   p->add_constraint((*$1));
   delete($1);
   $$=p;
}
;

linear_inequality: linexpression LEQ linexpression {
   $$= new Constraint((*$1) <= (*$3));
   delete($1);
   delete($3);
}
| linexpression GEQ linexpression{
   $$= new Constraint((*$1) >= (*$3));
   delete($1);
   delete($3);
}
| linexpression EQ linexpression {
   $$= new Constraint((*$1) == (*$3));
   delete($1);
   delete($3);
}
;


linexpression:  linexpression '+' linterm {
   (*$1)+= (*$3);
   delete($3);
   $$= $1;
}
|  linexpression '-' linterm{

   (*$1)= (*$1)-(*$3);
   delete($3);
   $$= $1;
}
| linterm {
   $$=$1;
}
| '-' linterm %prec uminus{
   $$=$2;
   (*$2)= -(*$2);
}
;

linterm: INT {
   $$= new Linear_Expression($1);
}
| varidentifier {
   $$=new Linear_Expression(Variable($1));
}
| INT '*' varidentifier {
   $$= new Linear_Expression( $1 * Variable($3));
}
;

varidentifier: IDENT {
   int i;
   if ( (i=find_variable($1)) == VAR_NOT_FOUND )
      yyerror("Variable "+ (*$1) + " not declared ");

   delete($1);
   $$=i;
}
;

locidentifier: IDENT {
   if (!location_exists($1)) {
      yyerror("Location " + (*$1) + " does not exist ");
   }

   
   $$=$1;
}
;

updates: updates update{
   $$=$1;
   $$->push_back((*$2));
   delete($2);
}
| update{
   $$= new vector<ILEPair>();
   $$->push_back((*$1));
   delete($1);
}
;

update: varidentifier ASSIGN linexpression {
   $$= new ILEPair($1,(*$3));
   delete($3);
} ;

polyvariable: IDENT{
   if (!poly_var_exists($1)){
      yyerror(" Polyhedron "+(*$1)+" not defined");
   }
   
   $$=$1;
}
;
%%

int main( int argc, char * argv[]){
   parseinit(argc,argv);
   yyparse();

   parse_command_line(argc,argv);
   

   //dispatch
   
   if (propagate)
      ch79_invariant_generation();
   else
      LP_invariant_generation();
   
   
   return 1;
}

