
/*
 * LpInv: Prototype Program Analyzer for Linear Systems using 
 *        Linear Programming. 
 * Author: Sriram Sankanarayanan <srirams@theory.stanford.edu> 
 * Copyright (c) 2005 Sriram Sankaranarayanan.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *        
 */


#ifndef D__PARSE_GLOBALS__H_
#define D__PARSE_GLOBALS__H_

/*
 *
 * Define parser-independent globals so that things roll along smoothly
 *
 *
 */

#include <iostream>
#include <vector>
#include <string>
#include <ppl.hh>
#include "Rational.h"
#include "var-info.h"
#include "ResourceMonitor.h"
#include "LinExpr.h"
#include "Timer.h"
#include "TrLoc.h"
#include "TrRel.h"
#include "TrSys.h"
#include "TemplateBuilder.h"

using namespace std;
using namespace Parma_Polyhedra_Library;

/* Data */
extern var_info * f;
extern vector<TrLoc> * vlocs;
extern vector<TrRel> * vrels;
extern int dimension;
extern int linenum;
extern TrSys * global_system;
extern C_Polyhedron * final_poly;
extern vector<TemplateBuilder> glb_templates;


/* Parameters */
extern int prop_steps;
extern bool final_set;
extern int _strategy_id_;
extern bool propagate;
extern bool colon;
extern bool has_templates;
extern int n_enrichments;
extern int zap_size;
extern bool dz_widen;
extern bool dualize_guard;
extern int ncars;

/* Statistics */
extern int num_equations_before_enrich;
extern int max_dimension_lp;
extern long tot_dimension_lp;
extern int n_lin_program_calls;
extern int n_simplex_iterations;
extern long time_lin_programming;
extern long time_lp_invariants;

extern ResourceMonitor mem_monitor;

typedef struct{

   long total_time; // the total time
   long prop_time; // the time for the initial propagation 
   long widen_time; // the time for widening
   long generator_extract_time; // the time to extract generators through lr
   long mem_maxrss; // the maximum resident set size -- from /proc/self/statm
   long mem_idrss; // unshared data
   long mem_isrss;// unshared stack
   
   int mult_dims; // the number of multiplier dimensions

} cs_statistics;

typedef struct{

   long total_time; // the total time
   long prop_time; // the time for the initial propagation 
   long widen_time; // the time for widening
   
   long mem_maxrss; // the maximum resident set size -- from /proc/self/statm

} ch79_statistics;


extern cs_statistics cs_stats;

extern ch79_statistics ch79_stats;


/*
 * Parse the arguments and initialize
 *
 */

extern void parseinit(int argc, char * argv[]);

/*
 * Header has been read in. Initialize the dimension
 * variable
 */
extern void process_header();

/*
 * Find variable id
 */


extern int find_variable(string * what);

/*
 * Location exists? Get reference?
 *
 */

extern bool location_exists(string * what);

extern TrLoc & location_reference(string * what);
extern TrLoc & location_reference();

/*
 * add updates to transitions
 */

extern void add_updates_to_transitions(TrRel & what, vector<ILEPair> * upds);

extern int yyerror();
extern int yyerror(char * what);
extern int  yyerror(string what);


extern void make_system();

extern void LP_invariant_generation();
extern void print_stats_lp();
extern void make_system_and_propagate();
extern void ch79_invariant_generation();
extern void call_cs_inv();
extern void print_stats_cs();
extern void print_stats_ch79();


extern map<string,C_Polyhedron> * poly_var_map;

extern bool poly_var_exists(string * what);
extern C_Polyhedron & get_polyhedron(string * what);
extern void add_poly_definition( string * name, C_Polyhedron * what);

extern void get_mem_usage();

/*
 * Parse the command line
 */

extern void options_search_and_action(char * what);
extern void parse_command_line(int argc, char * argv[] );




#endif
